/*
 *author: Heiko Barg, Roman Klinger
 *date: 05.03.2002
 */

import javax.swing.*;
import javax.swing.table.*;
import java.text.*;
import java.util.*;

public class MoTDManagement extends AbstractTableModel implements TableModel,
    Streamable {
    private MoTDTable myMoTDTable = null;

    // Datumsformat f?r Textanzeige
    public static final SimpleDateFormat DATEFORMAT =
        new SimpleDateFormat("dd.MM.yyyy kk:mm");
    
    /** Initialisiert die Benutzerverwaltung */
    public MoTDManagement(/*IOControl ioCtl*/) {
        // streamIn(ioCtl);
    }
    
    /** Klassen Methoden */
    /** appendMoTD() erzeugt ein neues MotD Objekt und h?ngt es an
     * Pre: MoTDTable Objekt muss existieren
     * Post: ein neues MoTD Objekt am Ende des MoTDTable Vectors
     */
    public boolean appendMoTD(String author, String eMail, String date,
        String title, String text) {
        return myMoTDTable.appendMoTD(author, eMail, date, title, text);
    }

    /** deleteMoTD() l?scht ein MotD-Objekt anhand des Indexes
     * @precondition  MoTDTable-Objekt muss existieren
     * @postcondition Falls zu dem ?bergebenen Index ein MoTD Objekt existierte,
     *                wurde der Verweis aus dem Vector entfernt
     */
    public boolean deleteMoTD(int index) {
        return myMoTDTable.remove(index) != null;
    }

    /** StreamableInterface */
    public void streamIn(IOControl ioCtrl) {
        myMoTDTable = (MoTDTable) ioCtrl.readMoTDs();
        if (myMoTDTable == null) {
            myMoTDTable = new MoTDTable();
        }
    }
    
    public void streamOut(IOControl ioCtrl) {
        ioCtrl.writeMoTDs(myMoTDTable);
    }

    /** TableModel Interface (siehe JDK 1.4.0) */
    static final String[] columnNames = {
        "Titel", "Autor", "Email", "Datum"
    };
    
    public String getColumnName(int col) { 
        return columnNames[col]; 
    }
    
    public int getRowCount() { return myMoTDTable.size(); }
    public int getColumnCount() { return columnNames.length; }
    
    public boolean isCellEditable(int row, int col) {
        return (col == 0) || (col == 4);
    }
    
    public Object getValueAt(int row, int col) {
        if (row < 0 || row >= myMoTDTable.size()) return null;
        
        MoTD moTD = (MoTD) myMoTDTable.elementAt(row);
        
        switch (col) {
            case 0: return moTD.getTitle();
            case 1: return moTD.getAuthor();
            case 2: return moTD.getEMail();
            case 3: return moTD.getDate();
            case 4: return moTD.getText();
            default: return null;
        }
    }
    
    public void setValueAt(Object value, int row, int col) {
        if (row < 0 || row >= myMoTDTable.size()) return;
        
        MoTD moTD = (MoTD) myMoTDTable.elementAt(row);
        
        switch (col) {
            case 0:
                moTD.setTitle((String) value);
                break;
            case 1:
                moTD.setAuthor((String) value);
                break;
            case 2:
                moTD.setEMail((String) value);
                break;
            case 3:
                moTD.setDate((String) value);
                break;
            case 4:
                moTD.setText((String) value);
                break;
        }
    }
}
