/*
 * SoPraContentManagement.java
 *
 * Created on 27. Februar 2002, 00:22
 */

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

/**
 * @author Christian Blichmann
 */
public class SoPraContentManagement extends JFrame {
    // Interne Klassenattribute
    private IOControl myIOControl;
    
    private UserManagement myUserManagement;
    private MoTDManagement myMoTDManagement;
    private FAQManagement myFAQManagement;
    
    // Interne Variablen
    private String path;     // Haelt den Pfad zum aktuellen Programmverzeichnis
    
    private String curUserLogin;
    private String curUserFullName;                        // Vollst?ndiger Name
    private boolean curUserIsAdmin = false;                            // Rechte
    private String curUserEMail;                                        // Email

    private int contentRow = -1;
    private int listRow = -1;
    private boolean isFAQEditing = false;                   // FAQ wird editiert
    private boolean ignoreEvents = false;
    private boolean treeActive = false;
    
    // See: http://java.sun.com/xml/jaxp/dist/1.1/docs/tutorial/TOC.html#intro
    //private HTMLGenerator myHTMLGenerator;
    
    /**
     * pre: In der Kommandozeile muss der Pfad zum Programmverzeichnis angegeben werden
     * post: Kreiert und startet die GUI
     */
    public static void main(String[] args) {
        // Starten der GUI
        SoPraContentManagement myMainFrame = new SoPraContentManagement(args);
        myMainFrame.show();
    }
    
    /** Creates new form JFrame */
    public SoPraContentManagement(String[] args) {
        // Pfad aus Kommandozeile, Workaround f?r Forte
        // (/home/sopgr15/sopr157/projekte/forte)
        path = args.length == 1 ? args[0] : ".";
        
        // Splashscreen anzeigen
        JAppSplashScreen mySplashScreen = new JAppSplashScreen(this);
        mySplashScreen.show();
        
        // Vorherige Instanz ?berpr?fen, benutzt einfaches File-locking
        doCheckPrevInstance();
        
        // Control-Klassen instantiieren
        myIOControl = new IOControl(path);
        myUserManagement = new UserManagement();
        myMoTDManagement = new MoTDManagement();
        myFAQManagement = new FAQManagement();

        // Alles Laden
        myUserManagement.streamIn(myIOControl);
        loadContent();
        
        doLogin();                    // Dem Benutzer erlauben, sich einzuloggen
        
        // Komponenten des Hauptfensters initialisieren
        initComponents();
        
        // Event Listener installieren (leider keine Forte-Unterst?tzung)
        ListViewTable.getSelectionModel().addListSelectionListener(
            new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
                ListViewTableSelectionChanged(evt);
            }
        });

        // Den JTree anpassen
        doSetupContentsTree();
       
        // Benutzerrechte anwenden (Men?s deaktivieren, usw.)
        doApplyUserRights();

        // Anwendung geladen, Splashscreen schlie?en
        mySplashScreen.setVisible(false);
        mySplashScreen.dispose();
    }
    
    private void doCheckPrevInstance() {
        File lockFile = new File(path + File.separator + ".lock");

        try {
            if (!lockFile.createNewFile())
            if (JOptionPane.showConfirmDialog(null, "Es wird bereits eine " +
                    "andere Instanz von SoPra Content-Management ausgef?hrt.\n" +
                    "Dies kann zu Datenverlust f?hren, falls beide " +
                    "Instanzen auf die gleichen Dateien zugreifen.\n" + 
                    "Klicken Sie auf \"Ok\", um fortzusetzen, oder auf " +
                    "\"Abbrechen\", um das Programm zu beenden.",
                    "SoPra Content-Management", JOptionPane.OK_CANCEL_OPTION) ==
                    JOptionPane.CANCEL_OPTION
            ) System.exit(0);
            
            lockFile.deleteOnExit();
        }
        catch (Exception e) {
            System.err.println("<" + e.getClass() + ">: "+ e.getMessage());
            System.exit(1);
        }
    }

    private void loadContent() {
        myMoTDManagement.streamIn(myIOControl);
        myFAQManagement.streamIn(myIOControl);
    }
    
    private void saveContent() {
        myMoTDManagement.streamOut(myIOControl);
        myFAQManagement.streamOut(myIOControl);
    }
    
    private void doLogin() {
        boolean myLoginSucceeded;

        do {
            JAppLoginDialog myLoginDialog = new JAppLoginDialog(null);
            myLoginDialog.show();
            
            myLoginSucceeded = myUserManagement.loginUser(
                curUserLogin = myLoginDialog.getLoginName(),
                myLoginDialog.getPassword());
            
            if (!myLoginSucceeded)
                JOptionPane.showMessageDialog(null,
                    "Sie konnten nicht am System angemeldet werden, da Sie\n" +
                    "entweder Ihren Loginnamen oder Ihr Passwort falsch \n" +
                    "eingegeben haben.\n" +
                    "Stellen Sie sicher, dass die FESTSTELLTASTE deaktiviert\n" +
                    "ist, und versuchen Sie es noch einmal.",
                    "SoPra Content-Management", JOptionPane.INFORMATION_MESSAGE);
        } while (!myLoginSucceeded);
        
        curUserIsAdmin = myUserManagement.isAdminUser(curUserLogin);
        curUserFullName = myUserManagement.getFullNameUser(curUserLogin);
        curUserEMail = myUserManagement.getEMailUser(curUserLogin);
        //*/
 
        /*myUserManagement.loginUser("tequilla", "sunrise");

        curUserLogin = "tequilla";
        curUserIsAdmin = true;
        curUserFullName = "Speedy Gonzales";
        curUserEMail = "speedy@gonzales.net";
        //*/
    }
    
    private void doSetupContentsTree() {
        // Standardmodell setzen
        ContentsTree.setModel(myFAQManagement);
        
        // Erscheinungsbild des JTree anpassen
        ContentsTree.putClientProperty("JTree.lineStyle", "Angled");
        ContentsTree.getSelectionModel().setSelectionMode(     // Einfachauswahl
            TreeSelectionModel.SINGLE_TREE_SELECTION);
        
        // Icons des JTree etwas anpassen
        DefaultTreeCellRenderer tcr = new DefaultTreeCellRenderer();
        ImageIcon ti = new ImageIcon(path + File.separator + "images" +
            File.separator + "icon_treetopic.gif");

        tcr.setLeafIcon(ti);     // Da wir keine "Bl?tter" im eigentlichen Sinne
        tcr.setClosedIcon(ti);  // haben, nur bei setOpenIcon() ein anderes Icon
        tcr.setOpenIcon(new ImageIcon(path + File.separator + "images" +
            File.separator + "icon_treetopicopen.gif"));
        ContentsTree.setCellRenderer(tcr);

        // MoTD als Standard ausw?hlen
        ContentsTree.setSelectionRow(0);
        
        // Drag and Drop
        //ContentsTree.setTransferHandler(BasicTreeUI.TreeTransferHandler);
    }
    
    private void doApplyUserRights() {
        // Den FAQ-Baum darf man nur als Verwalter editieren
        myFAQManagement.setAdminUser(curUserIsAdmin);
        ContentsTree.setEditable(curUserIsAdmin);

        // Diese Men?eintr?ge bekommt man nur als Verwalter zu Gesicht
        ToolsAddFAQTopicMenuItem.setEnabled(curUserIsAdmin);
        ToolsRenameFAQTopicMenuItem.setEnabled(curUserIsAdmin);
        ToolsDeleteFAQTopicMenuItem.setEnabled(curUserIsAdmin);
        ToolsAddFAQMenuItem.setEnabled(curUserIsAdmin);
        ToolsDeleteFAQMenuItem.setEnabled(curUserIsAdmin);
        ToolsUserManagementMenuItem.setEnabled(curUserIsAdmin);
        
        // Ebenso diese Symbolleiste
        FAQToolBar.setVisible(curUserIsAdmin);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     * @precondition Bring mir ein Gebuesch - nicht zu gross und nicht zu klein
     * @postcondition GUI-Komponenten wurden initialisiert
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        MainFrameMenuBar = new javax.swing.JMenuBar();
        FileMenu = new javax.swing.JMenu();
        FileCreateHTML = new javax.swing.JMenuItem();
        FileSeparator1 = new javax.swing.JSeparator();
        FileExitMenuItem = new javax.swing.JMenuItem();
        ToolsMenu = new javax.swing.JMenu();
        ToolsAddMoTDMenuItem = new javax.swing.JMenuItem();
        ToolsDeleteMoTDMenuItem = new javax.swing.JMenuItem();
        ToolsSeparator1 = new javax.swing.JSeparator();
        ToolsAddFAQTopicMenuItem = new javax.swing.JMenuItem();
        ToolsRenameFAQTopicMenuItem = new javax.swing.JMenuItem();
        ToolsDeleteFAQTopicMenuItem = new javax.swing.JMenuItem();
        ToolsSeparator2 = new javax.swing.JSeparator();
        ToolsAddFAQMenuItem = new javax.swing.JMenuItem();
        ToolsDeleteFAQMenuItem = new javax.swing.JMenuItem();
        ToolsSeparator3 = new javax.swing.JSeparator();
        ToolsUserManagementMenuItem = new javax.swing.JMenuItem();
        HelpMenu = new javax.swing.JMenu();
        HelpAboutMenuItem = new javax.swing.JMenuItem();
        MainFrameInteriorPanel = new javax.swing.JPanel();
        MainToolBar = new javax.swing.JPanel();
        MoTDToolBar = new javax.swing.JToolBar();
        AddMoTDButton = new javax.swing.JButton();
        DeleteMoTDButton = new javax.swing.JButton();
        FAQToolBar = new javax.swing.JToolBar();
        AddFAQTopicButton = new javax.swing.JButton();
        RenameFAQTopicButton = new javax.swing.JButton();
        DeleteFAQTopicButton = new javax.swing.JButton();
        FAQSeparator1 = new javax.swing.JSeparator();
        AddFAQButton = new javax.swing.JButton();
        DeleteFAQButton = new javax.swing.JButton();
        OrganizeFAQToolBar = new javax.swing.JToolBar();
        FAQUpButton = new javax.swing.JButton();
        FAQDownButton = new javax.swing.JButton();
        OrganizeFAQSeparator1 = new javax.swing.JSeparator();
        FAQLeftButton = new javax.swing.JButton();
        FAQRightButton = new javax.swing.JButton();
        MainFrameInteriorSplitPane = new javax.swing.JSplitPane();
        WorkAreaSplitPane = new javax.swing.JSplitPane();
        ListViewScrollPane = new javax.swing.JScrollPane();
        ListViewTable = new javax.swing.JTable();
        EditorPanel = new javax.swing.JPanel();
        EditorDetailPanel = new javax.swing.JPanel();
        TitleDetailDescLabel = new javax.swing.JLabel();
        TitleDetailLabel = new javax.swing.JLabel();
        AuthorDetailDescLabel = new javax.swing.JLabel();
        AuthorDetailLabel = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        EditorTextPane = new javax.swing.JTextPane();
        ContentsTree = new javax.swing.JTree();
        
        MainFrameMenuBar.setBackground(new java.awt.Color(212, 208, 200));
        FileMenu.setBackground(new java.awt.Color(212, 208, 200));
        FileMenu.setText("Datei");
        FileMenu.setFont(new java.awt.Font("Dialog", 0, 11));
        FileCreateHTML.setBackground(new java.awt.Color(212, 208, 200));
        FileCreateHTML.setFont(new java.awt.Font("Dialog", 0, 11));
        FileCreateHTML.setText("HTML Erstellen");
        FileCreateHTML.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_createhtml.gif")));
        FileCreateHTML.setPreferredSize(new java.awt.Dimension(151, 20));
        FileCreateHTML.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FileCreateHTMLActionPerformed(evt);
            }
        });
        
        FileMenu.add(FileCreateHTML);
        FileSeparator1.setBackground(new java.awt.Color(212, 208, 200));
        FileMenu.add(FileSeparator1);
        FileExitMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        FileExitMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        FileExitMenuItem.setText("Beenden");
        FileExitMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_transparent.gif")));
        FileExitMenuItem.setPreferredSize(new java.awt.Dimension(151, 20));
        FileExitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FileExitMenuItemActionPerformed(evt);
            }
        });
        
        FileMenu.add(FileExitMenuItem);
        MainFrameMenuBar.add(FileMenu);
        ToolsMenu.setBackground(new java.awt.Color(212, 208, 200));
        ToolsMenu.setText("Tools");
        ToolsMenu.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsAddMoTDMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsAddMoTDMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsAddMoTDMenuItem.setText("MoTD-Eintrag Hinzuf\u00fcgen");
        ToolsAddMoTDMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_addmotd.gif")));
        ToolsAddMoTDMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsAddMoTDMenuItem.setEnabled(false);
        ToolsAddMoTDMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsAddMoTDMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsAddMoTDMenuItem);
        ToolsDeleteMoTDMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsDeleteMoTDMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsDeleteMoTDMenuItem.setText("MoTD-Eintrag L\u00f6schen");
        ToolsDeleteMoTDMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_deletemotd.gif")));
        ToolsDeleteMoTDMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsDeleteMoTDMenuItem.setEnabled(false);
        ToolsDeleteMoTDMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsDeleteMoTDMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsDeleteMoTDMenuItem);
        ToolsSeparator1.setBackground(new java.awt.Color(212, 208, 200));
        ToolsMenu.add(ToolsSeparator1);
        ToolsAddFAQTopicMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsAddFAQTopicMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsAddFAQTopicMenuItem.setText("FAQ-Thema Hinzuf\u00fcgen");
        ToolsAddFAQTopicMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_addfaqtopic.gif")));
        ToolsAddFAQTopicMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsAddFAQTopicMenuItem.setEnabled(false);
        ToolsAddFAQTopicMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsAddFAQTopicMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsAddFAQTopicMenuItem);
        ToolsRenameFAQTopicMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsRenameFAQTopicMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsRenameFAQTopicMenuItem.setText("FAQ-Thema Umbenennen");
        ToolsRenameFAQTopicMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_faqrename.gif")));
        ToolsRenameFAQTopicMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsRenameFAQTopicMenuItem.setEnabled(false);
        ToolsRenameFAQTopicMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsRenameFAQTopicMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsRenameFAQTopicMenuItem);
        ToolsDeleteFAQTopicMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsDeleteFAQTopicMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsDeleteFAQTopicMenuItem.setText("FAQ-Thema L\u00f6schen");
        ToolsDeleteFAQTopicMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_deletefaqtopic.gif")));
        ToolsDeleteFAQTopicMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsDeleteFAQTopicMenuItem.setEnabled(false);
        ToolsDeleteFAQTopicMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsDeleteFAQTopicMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsDeleteFAQTopicMenuItem);
        ToolsSeparator2.setBackground(new java.awt.Color(212, 208, 200));
        ToolsMenu.add(ToolsSeparator2);
        ToolsAddFAQMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsAddFAQMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsAddFAQMenuItem.setText("FAQ Hinzuf\u00fcgen");
        ToolsAddFAQMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_addfaq.gif")));
        ToolsAddFAQMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsAddFAQMenuItem.setEnabled(false);
        ToolsAddFAQMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsAddFAQMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsAddFAQMenuItem);
        ToolsDeleteFAQMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsDeleteFAQMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsDeleteFAQMenuItem.setText("FAQ L\u00f6schen");
        ToolsDeleteFAQMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_deletefaq.gif")));
        ToolsDeleteFAQMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsDeleteFAQMenuItem.setEnabled(false);
        ToolsDeleteFAQMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsDeleteFAQMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsDeleteFAQMenuItem);
        ToolsSeparator3.setBackground(new java.awt.Color(212, 208, 200));
        ToolsMenu.add(ToolsSeparator3);
        ToolsUserManagementMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        ToolsUserManagementMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        ToolsUserManagementMenuItem.setText("Benutzerverwaltung...");
        ToolsUserManagementMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_usermanagement.gif")));
        ToolsUserManagementMenuItem.setPreferredSize(new java.awt.Dimension(173, 20));
        ToolsUserManagementMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ToolsUserManagementMenuItemActionPerformed(evt);
            }
        });
        
        ToolsMenu.add(ToolsUserManagementMenuItem);
        MainFrameMenuBar.add(ToolsMenu);
        HelpMenu.setBackground(new java.awt.Color(212, 208, 200));
        HelpMenu.setText("Hilfe");
        HelpMenu.setFont(new java.awt.Font("Dialog", 0, 11));
        HelpAboutMenuItem.setBackground(new java.awt.Color(212, 208, 200));
        HelpAboutMenuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        HelpAboutMenuItem.setText("Info...");
        HelpAboutMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_transparent.gif")));
        HelpAboutMenuItem.setPreferredSize(new java.awt.Dimension(75, 20));
        HelpAboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                HelpAboutMenuItemActionPerformed(evt);
            }
        });
        
        HelpMenu.add(HelpAboutMenuItem);
        MainFrameMenuBar.add(HelpMenu);
        
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("SoPra Content-Management");
        setBackground(new java.awt.Color(212, 208, 200));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        
        MainFrameInteriorPanel.setLayout(new java.awt.BorderLayout());
        
        MainToolBar.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 0, 0));
        
        MainToolBar.setBackground(new java.awt.Color(212, 208, 200));
        MoTDToolBar.setBackground(new java.awt.Color(212, 208, 200));
        AddMoTDButton.setBackground(new java.awt.Color(212, 208, 200));
        AddMoTDButton.setFont(new java.awt.Font("Dialog", 0, 11));
        AddMoTDButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_addmotd.gif")));
        AddMoTDButton.setToolTipText("MoTD hinzuf\u00fcgen");
        AddMoTDButton.setMaximumSize(new java.awt.Dimension(56, 27));
        AddMoTDButton.setMinimumSize(new java.awt.Dimension(56, 27));
        AddMoTDButton.setPreferredSize(new java.awt.Dimension(25, 25));
        AddMoTDButton.setEnabled(false);
        AddMoTDButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                AddMoTDButtonActionPerformed(evt);
            }
        });
        
        MoTDToolBar.add(AddMoTDButton);
        
        DeleteMoTDButton.setBackground(new java.awt.Color(212, 208, 200));
        DeleteMoTDButton.setFont(new java.awt.Font("Dialog", 0, 11));
        DeleteMoTDButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_deletemotd.gif")));
        DeleteMoTDButton.setToolTipText("MoTD l\u00f6schen");
        DeleteMoTDButton.setMaximumSize(new java.awt.Dimension(56, 27));
        DeleteMoTDButton.setMinimumSize(new java.awt.Dimension(56, 27));
        DeleteMoTDButton.setPreferredSize(new java.awt.Dimension(25, 25));
        DeleteMoTDButton.setEnabled(false);
        DeleteMoTDButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                DeleteMoTDButtonActionPerformed(evt);
            }
        });
        
        MoTDToolBar.add(DeleteMoTDButton);
        
        MainToolBar.add(MoTDToolBar);
        
        FAQToolBar.setBackground(new java.awt.Color(212, 208, 200));
        AddFAQTopicButton.setBackground(new java.awt.Color(212, 208, 200));
        AddFAQTopicButton.setFont(new java.awt.Font("Dialog", 0, 11));
        AddFAQTopicButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_addfaqtopic.gif")));
        AddFAQTopicButton.setToolTipText("FAQ-Thema hinzuf\u00fcgen");
        AddFAQTopicButton.setPreferredSize(new java.awt.Dimension(25, 25));
        AddFAQTopicButton.setEnabled(false);
        AddFAQTopicButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                AddFAQTopicButtonActionPerformed(evt);
            }
        });
        
        FAQToolBar.add(AddFAQTopicButton);
        
        RenameFAQTopicButton.setBackground(new java.awt.Color(212, 208, 200));
        RenameFAQTopicButton.setFont(new java.awt.Font("Dialog", 0, 11));
        RenameFAQTopicButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_faqrename.gif")));
        RenameFAQTopicButton.setToolTipText("FAQ-Thema umbenennen");
        RenameFAQTopicButton.setPreferredSize(new java.awt.Dimension(25, 25));
        RenameFAQTopicButton.setEnabled(false);
        RenameFAQTopicButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                RenameFAQTopicButtonActionPerformed(evt);
            }
        });
        
        FAQToolBar.add(RenameFAQTopicButton);
        
        DeleteFAQTopicButton.setBackground(new java.awt.Color(212, 208, 200));
        DeleteFAQTopicButton.setFont(new java.awt.Font("Dialog", 0, 11));
        DeleteFAQTopicButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_deletefaqtopic.gif")));
        DeleteFAQTopicButton.setToolTipText("FAQ-Thema l\u00f6schen");
        DeleteFAQTopicButton.setPreferredSize(new java.awt.Dimension(25, 25));
        DeleteFAQTopicButton.setEnabled(false);
        DeleteFAQTopicButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                DeleteFAQTopicButtonActionPerformed(evt);
            }
        });
        
        FAQToolBar.add(DeleteFAQTopicButton);
        
        FAQSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        FAQSeparator1.setPreferredSize(new java.awt.Dimension(6, 0));
        FAQToolBar.add(FAQSeparator1);
        
        AddFAQButton.setBackground(new java.awt.Color(212, 208, 200));
        AddFAQButton.setFont(new java.awt.Font("Dialog", 0, 11));
        AddFAQButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_addfaq.gif")));
        AddFAQButton.setToolTipText("FAQ hinzuf\u00fcgen");
        AddFAQButton.setPreferredSize(new java.awt.Dimension(25, 25));
        AddFAQButton.setEnabled(false);
        AddFAQButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                AddFAQButtonActionPerformed(evt);
            }
        });
        
        FAQToolBar.add(AddFAQButton);
        
        DeleteFAQButton.setBackground(new java.awt.Color(212, 208, 200));
        DeleteFAQButton.setFont(new java.awt.Font("Dialog", 0, 11));
        DeleteFAQButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_deletefaq.gif")));
        DeleteFAQButton.setToolTipText("FAQ l\u00f6schen");
        DeleteFAQButton.setPreferredSize(new java.awt.Dimension(25, 25));
        DeleteFAQButton.setEnabled(false);
        DeleteFAQButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                DeleteFAQButtonActionPerformed(evt);
            }
        });
        
        FAQToolBar.add(DeleteFAQButton);
        
        MainToolBar.add(FAQToolBar);
        
        FAQUpButton.setBackground(new java.awt.Color(212, 208, 200));
        FAQUpButton.setFont(new java.awt.Font("Dialog", 0, 11));
        FAQUpButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_arrowup.gif")));
        FAQUpButton.setToolTipText("FAQ-Eintrag/Thema nach oben");
        FAQUpButton.setPreferredSize(new java.awt.Dimension(25, 25));
        FAQUpButton.setEnabled(false);
        FAQUpButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FAQUpButtonActionPerformed(evt);
            }
        });
        
        OrganizeFAQToolBar.add(FAQUpButton);
        
        FAQDownButton.setBackground(new java.awt.Color(212, 208, 200));
        FAQDownButton.setFont(new java.awt.Font("Dialog", 0, 11));
        FAQDownButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_arrowdown.gif")));
        FAQDownButton.setToolTipText("FAQ-Eintrag/Thema nach unten");
        FAQDownButton.setPreferredSize(new java.awt.Dimension(25, 25));
        FAQDownButton.setEnabled(false);
        FAQDownButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FAQDownButtonActionPerformed(evt);
            }
        });
        
        OrganizeFAQToolBar.add(FAQDownButton);
        
        OrganizeFAQSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        OrganizeFAQSeparator1.setPreferredSize(new java.awt.Dimension(6, 0));
        OrganizeFAQToolBar.add(OrganizeFAQSeparator1);
        
        FAQLeftButton.setBackground(new java.awt.Color(212, 208, 200));
        FAQLeftButton.setFont(new java.awt.Font("Dialog", 0, 11));
        FAQLeftButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_arrowleft.gif")));
        FAQLeftButton.setToolTipText("FAQ-Thema nach links");
        FAQLeftButton.setPreferredSize(new java.awt.Dimension(25, 25));
        FAQLeftButton.setEnabled(false);
        FAQLeftButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FAQLeftButtonActionPerformed(evt);
            }
        });
        
        OrganizeFAQToolBar.add(FAQLeftButton);
        
        FAQRightButton.setBackground(new java.awt.Color(212, 208, 200));
        FAQRightButton.setFont(new java.awt.Font("Dialog", 0, 11));
        FAQRightButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_arrowright.gif")));
        FAQRightButton.setToolTipText("FAQ-Eintrag/Thema nach rechts");
        FAQRightButton.setPreferredSize(new java.awt.Dimension(25, 25));
        FAQRightButton.setEnabled(false);
        FAQRightButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FAQRightButtonActionPerformed(evt);
            }
        });
        
        OrganizeFAQToolBar.add(FAQRightButton);
        
        MainToolBar.add(OrganizeFAQToolBar);
        
        MainFrameInteriorPanel.add(MainToolBar, java.awt.BorderLayout.NORTH);
        
        MainFrameInteriorSplitPane.setBackground(new java.awt.Color(212, 208, 200));
        MainFrameInteriorSplitPane.setBorder(null);
        MainFrameInteriorSplitPane.setOneTouchExpandable(true);
        WorkAreaSplitPane.setBackground(new java.awt.Color(212, 208, 200));
        WorkAreaSplitPane.setBorder(null);
        WorkAreaSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        WorkAreaSplitPane.setOneTouchExpandable(true);
        ListViewScrollPane.setBackground(java.awt.Color.white);
        ListViewScrollPane.setBorder(null);
        ListViewScrollPane.setMinimumSize(new java.awt.Dimension(0, 0));
        ListViewScrollPane.setPreferredSize(new java.awt.Dimension(400, 160));
        ListViewTable.setFont(new java.awt.Font("Dialog", 0, 11));
        ListViewTable.setModel(new javax.swing.table.DefaultTableModel(
        new Object [][] {
            
        },
        new String [] {
            
        }
        ));
        ListViewTable.setDragEnabled(true);
        ListViewTable.setMaximumSize(new java.awt.Dimension(2147483647, 2147483647));
        ListViewTable.setMinimumSize(new java.awt.Dimension(0, 1));
        ListViewTable.setShowHorizontalLines(false);
        ListViewTable.setShowVerticalLines(false);
        ListViewTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        ListViewScrollPane.setViewportView(ListViewTable);
        
        WorkAreaSplitPane.setLeftComponent(ListViewScrollPane);
        
        EditorPanel.setLayout(new java.awt.BorderLayout());
        
        EditorPanel.setBackground(new java.awt.Color(212, 208, 200));
        EditorDetailPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        EditorDetailPanel.setBackground(new java.awt.Color(212, 208, 200));
        EditorDetailPanel.setMinimumSize(new java.awt.Dimension(0, 42));
        EditorDetailPanel.setPreferredSize(new java.awt.Dimension(0, 32));
        TitleDetailDescLabel.setFont(new java.awt.Font("Dialog", 1, 11));
        TitleDetailDescLabel.setText("Titel:");
        TitleDetailDescLabel.setMaximumSize(new java.awt.Dimension(64, 16));
        TitleDetailDescLabel.setMinimumSize(new java.awt.Dimension(64, 16));
        TitleDetailDescLabel.setPreferredSize(new java.awt.Dimension(64, 16));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 4, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        EditorDetailPanel.add(TitleDetailDescLabel, gridBagConstraints1);
        
        TitleDetailLabel.setFont(new java.awt.Font("Dialog", 0, 11));
        TitleDetailLabel.setText("(markieren Sie einen Eintrag um seinen Titel anzuzeigen)");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 2.0;
        EditorDetailPanel.add(TitleDetailLabel, gridBagConstraints1);
        
        AuthorDetailDescLabel.setFont(new java.awt.Font("Dialog", 1, 11));
        AuthorDetailDescLabel.setText("Autor:");
        AuthorDetailDescLabel.setMaximumSize(new java.awt.Dimension(64, 16));
        AuthorDetailDescLabel.setMinimumSize(new java.awt.Dimension(64, 16));
        AuthorDetailDescLabel.setPreferredSize(new java.awt.Dimension(64, 16));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 4, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        EditorDetailPanel.add(AuthorDetailDescLabel, gridBagConstraints1);
        
        AuthorDetailLabel.setFont(new java.awt.Font("Dialog", 0, 11));
        AuthorDetailLabel.setText("(markieren Sie einen Eintrag um seinen Autor anzuzeigen)");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 2.0;
        EditorDetailPanel.add(AuthorDetailLabel, gridBagConstraints1);
        
        EditorPanel.add(EditorDetailPanel, java.awt.BorderLayout.NORTH);
        
        jScrollPane1.setMinimumSize(new java.awt.Dimension(0, 0));
        EditorTextPane.setMinimumSize(new java.awt.Dimension(0, 0));
        EditorTextPane.setPreferredSize(new java.awt.Dimension(400, 160));
        EditorTextPane.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                EditorTextPaneKeyTyped(evt);
            }
        });
        
        jScrollPane1.setViewportView(EditorTextPane);
        
        EditorPanel.add(jScrollPane1, java.awt.BorderLayout.CENTER);
        
        WorkAreaSplitPane.setRightComponent(EditorPanel);
        
        MainFrameInteriorSplitPane.setRightComponent(WorkAreaSplitPane);
        
        ContentsTree.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        ContentsTree.setFont(new java.awt.Font("Dialog", 0, 11));
        ContentsTree.setMaximumSize(new java.awt.Dimension(2147483647, 2147483647));
        ContentsTree.setPreferredSize(new java.awt.Dimension(160, 72));
        ContentsTree.setRootVisible(false);
        ContentsTree.setShowsRootHandles(true);
        ContentsTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                ContentsTreeValueChanged(evt);
            }
        });
        
        MainFrameInteriorSplitPane.setLeftComponent(ContentsTree);
        
        MainFrameInteriorPanel.add(MainFrameInteriorSplitPane, java.awt.BorderLayout.CENTER);
        
        getContentPane().add(MainFrameInteriorPanel, java.awt.BorderLayout.CENTER);
        
        setJMenuBar(MainFrameMenuBar);
        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        java.awt.Dimension dialogSize = getSize();
        setLocation((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
    }//GEN-END:initComponents

    private void EditorTextPaneKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_EditorTextPaneKeyTyped
        doUpdateData();
    }//GEN-LAST:event_EditorTextPaneKeyTyped

    private void FAQRightButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FAQRightButtonActionPerformed
        if (!treeActive) {
            if (myFAQManagement.moveFAQRight(listRow)) {
                ListViewTable.updateUI();
            }
        } else {
            if (myFAQManagement.moveFAQTopicRight()) {
            }
        }
    }//GEN-LAST:event_FAQRightButtonActionPerformed

    private void FAQLeftButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FAQLeftButtonActionPerformed
        if (!treeActive) {
            if (myFAQManagement.moveFAQLeft(listRow)) {
                ListViewTable.updateUI();
            }
        } else {
            if (myFAQManagement.moveFAQTopicLeft()) {
            }
        }
    }//GEN-LAST:event_FAQLeftButtonActionPerformed

    private void FAQUpButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FAQUpButtonActionPerformed
        if (!treeActive) {
            if (myFAQManagement.moveFAQUp(listRow)) {
                listRow--;
                ListViewTable.getSelectionModel().setSelectionInterval(listRow, listRow);
                ListViewTable.updateUI();
            }
        } else {
            contentRow--;
            if (myFAQManagement.moveFAQTopicUp()) {
                ContentsTree.setSelectionInterval(contentRow, contentRow);
                ContentsTree.updateUI();
            }
        }
    }//GEN-LAST:event_FAQUpButtonActionPerformed

    private void FAQDownButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FAQDownButtonActionPerformed
        if (!treeActive) {
            if (myFAQManagement.moveFAQDown(listRow)) {
                listRow++;
                ListViewTable.getSelectionModel().setSelectionInterval(listRow, listRow);
                ListViewTable.updateUI();
            }
        } else {
            contentRow++;
            if (myFAQManagement.moveFAQTopicDown()) {
                ContentsTree.setSelectionInterval(contentRow, contentRow);
                ContentsTree.updateUI();
            }
        }
    }//GEN-LAST:event_FAQDownButtonActionPerformed

    // FAQ-Topic Routinen
    private void doAddFAQTopic() {
        myFAQManagement.appendFAQTopic("(Neues Thema)");
        ContentsTree.updateUI();
    }
    
    private void doRenameFAQTopic() {
        TreeSelectionModel tsm = ContentsTree.getSelectionModel();
        
        if (!tsm.isSelectionEmpty()) {              // Markierung ist nicht leer
            String newName = (String) JOptionPane.showInputDialog(null,
            "Neuer Titel:", "SoPra Content-Management",
            JOptionPane.QUESTION_MESSAGE, null, null,
            myFAQManagement.getFAQTopicTitle());
            
            if (newName != null && !newName.equals("")) {
                myFAQManagement.renameFAQTopic(newName);
                ContentsTree.updateUI();
            }
        }
    }
    
    private void doDeleteFAQTopic() {
        if (JOptionPane.showConfirmDialog(null, "M?chten Sie dieses FAQ-Thema " +
            "l?schen?\nAlle Unterthemen und Fragen werden ebenfalls gel?scht.",
            "SoPra Content-Management", JOptionPane.YES_NO_OPTION) ==
            JOptionPane.NO_OPTION) return;
        
        TreeSelectionModel tsm = ContentsTree.getSelectionModel();
        
        if (!tsm.isSelectionEmpty()) {              // Markierung ist nicht leer
            myFAQManagement.deleteFAQTopic();
            ContentsTree.updateUI();
        }
    }
    
    private void ToolsDeleteFAQMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsDeleteFAQMenuItemActionPerformed
        doDeleteFAQ();
    }//GEN-LAST:event_ToolsDeleteFAQMenuItemActionPerformed

    private void ToolsAddFAQMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsAddFAQMenuItemActionPerformed
        doAddFAQ();
    }//GEN-LAST:event_ToolsAddFAQMenuItemActionPerformed

    private void ToolsDeleteFAQTopicMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsDeleteFAQTopicMenuItemActionPerformed
        doDeleteFAQTopic();
    }//GEN-LAST:event_ToolsDeleteFAQTopicMenuItemActionPerformed

    private void ToolsRenameFAQTopicMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsRenameFAQTopicMenuItemActionPerformed
        doRenameFAQTopic();
    }//GEN-LAST:event_ToolsRenameFAQTopicMenuItemActionPerformed

    private void ToolsAddFAQTopicMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsAddFAQTopicMenuItemActionPerformed
        doAddFAQTopic();
    }//GEN-LAST:event_ToolsAddFAQTopicMenuItemActionPerformed

    // FAQ-Routinen
    private void doAddFAQ() {
        myFAQManagement.appendFAQ(curUserFullName, curUserEMail, "(Neue FAQ)",
            "(Text hier eingeben)");
        
        ListViewTable.updateUI();
    }
    
    private void doDeleteFAQ() {
        if (JOptionPane.showConfirmDialog(null, "Diesen FAQ-Eintrag l?schen?",
            "SoPra Content-Management", JOptionPane.YES_NO_OPTION) ==
            JOptionPane.NO_OPTION) return;
        
        ListSelectionModel lsm = ListViewTable.getSelectionModel();
        if (!lsm.isSelectionEmpty()) {              // Markierung ist nicht leer
            myFAQManagement.deleteFAQ(lsm.getMinSelectionIndex());
            ListViewTable.updateUI();
        }
    }
    
    // FAQ-ToolBar
    private void AddFAQTopicButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_AddFAQTopicButtonActionPerformed
        doAddFAQTopic();
    }//GEN-LAST:event_AddFAQTopicButtonActionPerformed

    private void RenameFAQTopicButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_RenameFAQTopicButtonActionPerformed
        doRenameFAQTopic();
    }//GEN-LAST:event_RenameFAQTopicButtonActionPerformed

    private void DeleteFAQTopicButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_DeleteFAQTopicButtonActionPerformed
        doDeleteFAQTopic();
    }//GEN-LAST:event_DeleteFAQTopicButtonActionPerformed

    // FAQ-Menue

    private void DeleteFAQButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_DeleteFAQButtonActionPerformed
        doDeleteFAQ();
    }//GEN-LAST:event_DeleteFAQButtonActionPerformed

    private void AddFAQButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_AddFAQButtonActionPerformed
        doAddFAQ();
    }//GEN-LAST:event_AddFAQButtonActionPerformed

    private void DeleteMoTDButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_DeleteMoTDButtonActionPerformed
        doDeleteMoTD();
    }//GEN-LAST:event_DeleteMoTDButtonActionPerformed

    private void AddMoTDButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_AddMoTDButtonActionPerformed
        doAddMoTD();
    }//GEN-LAST:event_AddMoTDButtonActionPerformed

    private void doDeleteMoTD() {
        if (JOptionPane.showConfirmDialog(null, "Diesen MoTD-Eintrag l\u00f6schen?",
            "SoPra Content-Management", JOptionPane.YES_NO_OPTION) ==
            JOptionPane.NO_OPTION) return;
        
        ListSelectionModel lsm = ListViewTable.getSelectionModel();
        if (!lsm.isSelectionEmpty()) {              // Markierung ist nicht leer
            myMoTDManagement.deleteMoTD(lsm.getMinSelectionIndex());
            ListViewTable.updateUI();
        }
    }
    
    private void ToolsDeleteMoTDMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsDeleteMoTDMenuItemActionPerformed
        doDeleteMoTD();
    }//GEN-LAST:event_ToolsDeleteMoTDMenuItemActionPerformed

    private void doUpdateUI() {
        // Men?s
        ToolsAddMoTDMenuItem.setEnabled(!isFAQEditing);
        ToolsDeleteMoTDMenuItem.setEnabled(!isFAQEditing);
        
        ToolsAddFAQTopicMenuItem.setEnabled(isFAQEditing && curUserIsAdmin);
        ToolsRenameFAQTopicMenuItem.setEnabled(isFAQEditing && curUserIsAdmin);
        ToolsDeleteFAQTopicMenuItem.setEnabled(isFAQEditing && curUserIsAdmin);
        
        ToolsAddFAQMenuItem.setEnabled(isFAQEditing && curUserIsAdmin);
        ToolsDeleteFAQMenuItem.setEnabled(isFAQEditing && curUserIsAdmin);
        
        // Toolbars
        OrganizeFAQToolBar.setVisible(isFAQEditing && curUserIsAdmin);

        AddMoTDButton.setEnabled(!isFAQEditing);
        DeleteMoTDButton.setEnabled(!isFAQEditing);
        
        AddFAQTopicButton.setEnabled(isFAQEditing && curUserIsAdmin);
        RenameFAQTopicButton.setEnabled(isFAQEditing && curUserIsAdmin);
        DeleteFAQTopicButton.setEnabled(isFAQEditing && curUserIsAdmin);
        
        AddFAQButton.setEnabled(isFAQEditing && curUserIsAdmin);
        DeleteFAQButton.setEnabled(isFAQEditing && curUserIsAdmin);
        
        // Editor
        doUpdateEditor();
    }
    
    private void doUpdateEditor() {
        String text = "";
        
        if (listRow >= 0) {
            if (isFAQEditing)
                text = (String) myFAQManagement.getValueAt(listRow, 3);
            else
                text = (String) myMoTDManagement.getValueAt(listRow, 4);
        }
        
        EditorTextPane.setText(text);
    }
    
    private void doUpdateData() {
        if (listRow >= 0) {
            if (isFAQEditing)
                myFAQManagement.setValueAt(EditorTextPane.getText(), listRow, 3);
            else
                myMoTDManagement.setValueAt(EditorTextPane.getText(), listRow, 4);
        }
/*        ListSelectionModel lsm = ListViewTable.getSelectionModel();
        if (lsm.isSelectionEmpty()) return;
        
        int selectedRow = lsm.getMinSelectionIndex();
        TableModel tm = ListViewTable.getModel();
        Object v;
        
        if (!isFAQEditing) {
            // Aenderungen in die MoTD-Tabelle ?bernehmen
            if (prevRow >= 0)
                myMoTDManagement.setValueAt(EditorTextPane.getText(),
                    prevRow, 4);

            v = myMoTDManagement.getValueAt(selectedRow, 4);
            EditorTextPane.setText(v != null ? (String) v : "");
        } else {
            // Aenderungen in die FAQ-Tabelle ?bernehmen
            if (prevRow >= 0)
                myFAQManagement.setValueAt(EditorTextPane.getText(),
                    prevRow, 3);

            v = myFAQManagement.getValueAt(selectedRow, 3);
            EditorTextPane.setText(v != null ? (String) v : "");
        }
 */
    }
    
    private void ContentsTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_ContentsTreeValueChanged
        boolean allowEdit = false;
        
        TreeSelectionModel tsm = ContentsTree.getSelectionModel();
        if (tsm.isSelectionEmpty()) return;
        
        contentRow = tsm.getMinSelectionRow();
        treeActive = true;
        isFAQEditing = contentRow >= 1;
        
        // MoTD- und FAQ-Nodes nicht editierbar
        ContentsTree.setEditable(allowEdit = contentRow > 1);
        
//        doUpdateData();

        int itemCount;
        if (isFAQEditing) {
            FAQNode node = (FAQNode) ContentsTree.getLastSelectedPathComponent();
            
            myFAQManagement.setCurTopic(node);
            ListViewTable.setModel(myFAQManagement);
            itemCount = myFAQManagement.getRowCount();
            
            EditorTextPane.setEditable(curUserIsAdmin);
            
            DefaultMutableTreeNode tmpNode = (DefaultMutableTreeNode) node.getParent();
            boolean allowMove = allowEdit && tmpNode != null;
            
            FAQUpButton.setEnabled(allowMove && tmpNode.getFirstChild() != node);
            FAQDownButton.setEnabled(allowMove && tmpNode.getLastChild() != node);
            
            FAQLeftButton.setEnabled(allowMove && tmpNode.getLevel() > 1);
            FAQRightButton.setEnabled(allowMove && tmpNode.getFirstChild() != node);
        } else {
            ListViewTable.setModel(myMoTDManagement);
            itemCount = myMoTDManagement.getRowCount();
            EditorTextPane.setEditable(true);
        }

        listRow = itemCount > 0 ? 0 : -1;
        ListViewTable.getSelectionModel().setSelectionInterval(0, 0);
        ListViewTable.updateUI();
        
        doUpdateUI();
        ListViewTable.updateUI();

        ToolsRenameFAQTopicMenuItem.setEnabled(allowEdit && curUserIsAdmin);
        ToolsDeleteFAQTopicMenuItem.setEnabled(allowEdit && curUserIsAdmin);

        RenameFAQTopicButton.setEnabled(allowEdit && curUserIsAdmin);
        DeleteFAQTopicButton.setEnabled(allowEdit && curUserIsAdmin);
        
/*        boolean allowEdit;
        ContentsTree.setEditable(        // MoTD- und FAQ-Nodes nicht editierbar
            allowEdit = ContentsTree.getSelectionModel().getMinSelectionRow() > 1);
        
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
            ContentsTree.getLastSelectedPathComponent();

        if (node == null) return;
        
        if (isFAQEditing = node instanceof FAQNode) {
            myFAQManagement.setCurTopic((FAQNode) node);
            ListViewTable.setModel(myFAQManagement);
            ListViewTable.updateUI();
        } else
            ListViewTable.setModel(myMoTDManagement);

        ListViewTable.getSelectionModel().setAnchorSelectionIndex(0);
        prevRow = -1;
        doUpdateUI();

        // Menues anpassen
        ToolsRenameFAQTopicMenuItem.setEnabled(allowEdit);
        ToolsDeleteFAQTopicMenuItem.setEnabled(allowEdit);
 */
    }//GEN-LAST:event_ContentsTreeValueChanged

    private void doAddMoTD() {
        myMoTDManagement.appendMoTD(curUserFullName, curUserEMail,
            MoTDManagement.DATEFORMAT.format(new Date()), "(Neue MoTD)",
            "(Text hier eingeben)");
        
        ListViewTable.updateUI();
    }
    
    private void ToolsAddMoTDMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsAddMoTDMenuItemActionPerformed
        doAddMoTD();
    }//GEN-LAST:event_ToolsAddMoTDMenuItemActionPerformed

    private void FileCreateHTMLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FileCreateHTMLActionPerformed
        JAppSplashScreen splash = new JAppSplashScreen(this);
        splash.setStatusText("HTML erzeugen...");
        splash.setVisible(true);
        
        saveContent();
        if (myIOControl.generateHTML())
            JOptionPane.showMessageDialog(null, "HTML Dateien erfolgreich erzeugt.");
        else
            JOptionPane.showMessageDialog(null, "HTML-Erzeugung fehlgeschlagen.");
        
        splash.setVisible(false);
        splash.dispose();
    }//GEN-LAST:event_FileCreateHTMLActionPerformed

    private void ToolsUserManagementMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ToolsUserManagementMenuItemActionPerformed
        JAppUserManagementDialog myUserManagementDialog =
            new JAppUserManagementDialog(this, myUserManagement, curUserLogin);
        
        myUserManagementDialog.show();
        curUserLogin = myUserManagementDialog.getCurLogin();
        myUserManagementDialog.dispose();
        myUserManagement.streamOut(myIOControl);
    }//GEN-LAST:event_ToolsUserManagementMenuItemActionPerformed

    private void ListViewTableSelectionChanged(ListSelectionEvent evt) {
        if (evt != null && evt.getValueIsAdjusting()) return;
        
        ListSelectionModel lsm = (ListSelectionModel) evt.getSource();
        if (!lsm.isSelectionEmpty()) {
            treeActive = false;
            
            TableModel tm = ListViewTable.getModel();

            int tmpRow = lsm.getMinSelectionIndex();
            if (tmpRow >= 0) {
                // GUI aktualisieren
                TitleDetailLabel.setText((String) tm.getValueAt(tmpRow, 0));
                AuthorDetailLabel.setText((String) tm.getValueAt(tmpRow, 1));
                
                //doUpdateData();
            }
            
            if (isFAQEditing) {
                FAQUpButton.setEnabled(tmpRow > 0);
                FAQDownButton.setEnabled(tmpRow < myFAQManagement.getRowCount() - 1);
                
                FAQLeftButton.setEnabled(true);
                FAQRightButton.setEnabled(true);
            }
            
            listRow = tmpRow;
            doUpdateEditor();
        } else {
            listRow = -1;
            TitleDetailLabel.setText("(markieren Sie einen Eintrag um seinen Titel anzuzeigen)");
            AuthorDetailLabel.setText("(markieren Sie einen Eintrag um seinen Autor anzuzeigen)");
            EditorTextPane.setText("");

            FAQLeftButton.setEnabled(false);
            FAQRightButton.setEnabled(false);
        }

/*        // ?berfl?ssige Ereignisse ignorieren
        if (evt.getValueIsAdjusting()) return;

        // Aktuelle Markierung holen (fuer zukuenftigen Gebrauch)
        ListSelectionModel lsm = (ListSelectionModel) evt.getSource();
        if (!lsm.isSelectionEmpty()) {              // Markierung ist nicht leer
            // Alternativ: int selectedRow = ListViewTable.getSelectedRow();
            int selectedRow = lsm.getMinSelectionIndex();
            TableModel tm = ListViewTable.getModel();
            
            // GUI aktualisieren
            TitleDetailLabel.setText(tm.getValueAt(selectedRow, 0).toString());
            AuthorDetailLabel.setText(tm.getValueAt(selectedRow, 1).toString());
            doUpdateUI();
            
            prevRow = selectedRow;
        } else
            prevRow = -1;
 */
    }
    
    private void FileExitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FileExitMenuItemActionPerformed
        doExitApplication();
    }//GEN-LAST:event_FileExitMenuItemActionPerformed

    private void doAboutDialog() {
        // Neuen About Dialog erzeugen und modal anzeigen
        JAppAboutDialog myAboutDialog = new JAppAboutDialog(this);
        myAboutDialog.show();
    }
        
    private void HelpAboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_HelpAboutMenuItemActionPerformed
        doAboutDialog();
    }//GEN-LAST:event_HelpAboutMenuItemActionPerformed

    /** Exit the Application */
    private void doExitApplication() {
        // Alles speichern
        myUserManagement.streamOut(myIOControl);
        saveContent();
        
        // Anwendung beenden
        System.exit(0);
    }

    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        doExitApplication();
    }//GEN-LAST:event_exitForm

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar MainFrameMenuBar;
    private javax.swing.JMenu FileMenu;
    private javax.swing.JMenuItem FileCreateHTML;
    private javax.swing.JSeparator FileSeparator1;
    private javax.swing.JMenuItem FileExitMenuItem;
    private javax.swing.JMenu ToolsMenu;
    private javax.swing.JMenuItem ToolsAddMoTDMenuItem;
    private javax.swing.JMenuItem ToolsDeleteMoTDMenuItem;
    private javax.swing.JSeparator ToolsSeparator1;
    private javax.swing.JMenuItem ToolsAddFAQTopicMenuItem;
    private javax.swing.JMenuItem ToolsRenameFAQTopicMenuItem;
    private javax.swing.JMenuItem ToolsDeleteFAQTopicMenuItem;
    private javax.swing.JSeparator ToolsSeparator2;
    private javax.swing.JMenuItem ToolsAddFAQMenuItem;
    private javax.swing.JMenuItem ToolsDeleteFAQMenuItem;
    private javax.swing.JSeparator ToolsSeparator3;
    private javax.swing.JMenuItem ToolsUserManagementMenuItem;
    private javax.swing.JMenu HelpMenu;
    private javax.swing.JMenuItem HelpAboutMenuItem;
    private javax.swing.JPanel MainFrameInteriorPanel;
    private javax.swing.JPanel MainToolBar;
    private javax.swing.JToolBar MoTDToolBar;
    private javax.swing.JButton AddMoTDButton;
    private javax.swing.JButton DeleteMoTDButton;
    private javax.swing.JToolBar FAQToolBar;
    private javax.swing.JButton AddFAQTopicButton;
    private javax.swing.JButton RenameFAQTopicButton;
    private javax.swing.JButton DeleteFAQTopicButton;
    private javax.swing.JSeparator FAQSeparator1;
    private javax.swing.JButton AddFAQButton;
    private javax.swing.JButton DeleteFAQButton;
    private javax.swing.JToolBar OrganizeFAQToolBar;
    private javax.swing.JButton FAQUpButton;
    private javax.swing.JButton FAQDownButton;
    private javax.swing.JSeparator OrganizeFAQSeparator1;
    private javax.swing.JButton FAQLeftButton;
    private javax.swing.JButton FAQRightButton;
    private javax.swing.JSplitPane MainFrameInteriorSplitPane;
    private javax.swing.JSplitPane WorkAreaSplitPane;
    private javax.swing.JScrollPane ListViewScrollPane;
    private javax.swing.JTable ListViewTable;
    private javax.swing.JPanel EditorPanel;
    private javax.swing.JPanel EditorDetailPanel;
    private javax.swing.JLabel TitleDetailDescLabel;
    private javax.swing.JLabel TitleDetailLabel;
    private javax.swing.JLabel AuthorDetailDescLabel;
    private javax.swing.JLabel AuthorDetailLabel;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextPane EditorTextPane;
    private javax.swing.JTree ContentsTree;
    // End of variables declaration//GEN-END:variables

}
